<?php

namespace WC_Integration_NettiX\Cache;

use DateTime;
use Exception;

class File_Cache implements Cache {

	private string $cache_name;
	private string $cache_content = "";
	private int $expire_time;

	public function __construct( string $cache_name, int $expire_time ) {
		$this->cache_name  = $cache_name;
		$this->expire_time = $expire_time;
	}

	/**
	 * @return string
	 */
	public function get_content(): string {
		if ( $this->cache_content === "" ) {
			$this->cache_content = $this->fetch_content();
		}

		return $this->cache_content;
	}

	/**
	 * @return string
	 */
	private function fetch_content(): string {
		try {
			$cache_dir = $this->get_cache_directory();

			if ( ! $cache_data = @file_get_contents( $cache_dir . '/' . $this->cache_name ) ) {
				return "";
			}

			$cache_data = unserialize( $cache_data );

			if ( ( new DateTime( $cache_data['expire_time_stamp'] ?? '' ) ) <= ( new DateTime ) ) {
				return "";
			}

			if ( ( $cache_data['hash'] ?? '' ) != $this->get_hash( $cache_data['content'] ?? null ) ) {
				return "";
			}

			return $cache_data['content'] ?? "";
		} catch ( Exception $e ) {
			return "";
		}
	}

	/**
	 * @param $content
	 *
	 * @return bool
	 */
	public function store_content( $content ): bool {
		try {
			$this->cache_content = $content;

			$cacheDir  = $this->get_cache_directory();
			$cacheData = [
				'content'           => $this->cache_content,
				'expire_time_stamp' => $this->get_current_expire_date_time()->format( 'Y-m-d H:i:s' ),
				'hash'              => $this->get_hash( $content ),
			];
			if ( file_put_contents( $cacheDir . '/' . $this->cache_name, serialize( $cacheData ) ) === false ) {
				return false;
			}
		} catch ( Exception $e ) {
			return false;
		}

		return true;
	}

	/**
	 * @return string
	 * @throws Exception
	 */
	private function get_cache_directory(): string {
		if ( $dir = sys_get_temp_dir() ) {
			return $dir;
		}

		$dir = '/tmp';
		if ( file_exists( $dir ) ) {
			return $dir;
		}

		throw new Exception( "Could not find suitable cache directory" );
	}

	/**
	 * @return DateTime
	 * @throws Exception
	 */
	private function get_current_expire_date_time(): DateTime {
		$date_time = new DateTime;
		$date_time->modify( '+' . $this->expire_time . ' seconds' );

		return $date_time;
	}

	private function get_hash( $content ): string {
		return hash( 'sha256', serialize( [ 'content' => $content, 'secret' => NONCE_SALT . AUTH_SALT ] ) );
	}
}