<?php

namespace WC_Integration_NettiX\Plugin;

use Exception;
use WC_Product;
use WC_Product_External;
use WC_Product_Query;
use WP_Error;
use WP_Term;

abstract class Ad_Handler {
	protected array $terms_cache = [];
	protected Database_Settings $database_settings;

	public function __construct() {
		$this->database_settings = new Database_Settings();
	}

	abstract public function handle( array $response_array );

	/**
	 * @param string $category_slug
	 *
	 * @return WP_Term
	 * @throws Exception
	 */
	protected function get_category( string $category_slug ): WP_Term {
		if ( ! array_key_exists( $category_slug, $this->terms_cache ) ) {
			$term = get_term_by( 'slug', $category_slug, 'product_cat' );

			if ( ! ( $term instanceof WP_Term ) ) {
				throw new Exception( "Could not find term for category slug '" . $category_slug . "'" );
			}

			$this->terms_cache[ $category_slug ] = $term;
		}

		return $this->terms_cache[ $category_slug ];
	}

	/**
	 * Returns tag as WP_Term. If term doesn't exist it's created. Uses local terms cache.
	 *
	 * @param string $tag_name
	 *
	 * @return WP_Term|null
	 * @throws Exception
	 */
	protected function get_tag( string $tag_name ): ?WP_Term {
		if ( ! $tag_name ) {
			return null;
		}

		$tag_name_slug = $this->create_name_slug( $tag_name );
		$tag_name      = $this->sanitize_tag_name( $tag_name );

		if ( ! array_key_exists( $tag_name_slug, $this->terms_cache ) ) {
			if ( ! term_exists( $tag_name, 'product_tag' ) ) {
				$this->create_tag( $tag_name );
			}

			$res = get_term_by( 'slug', $tag_name_slug, 'product_tag' );

			if ( $res instanceof WP_Term ) {
				$this->terms_cache[ $tag_name_slug ] = $res;
			} else {
				return null;
			}
		}

		return $this->terms_cache[ $tag_name_slug ];
	}

	/**
	 * @param string $tag_name
	 *
	 * @return void
	 * @throws Exception
	 */
	protected function create_tag( string $tag_name ): void {
		$tag_name_slug = $this->create_name_slug( $tag_name );

		$res = wp_insert_term( $tag_name,
			'product_tag',
			[ 'slug' => $tag_name_slug, 'description' => 'Product tag for ' . $tag_name ] );

		if ( $res instanceof WP_Error ) {
			throw new Exception( "Failed to add product tag for '" . $tag_name . "'" );
		}
	}

	/**
	 * @param string $name
	 *
	 * @return string
	 */
	protected function create_name_slug( string $name ): string {
		return str_replace( [ "`", "´", "'", "\"", ",", ";", ":", ".", " " ],
			[ "", "", "", "", "", "", "", "-", "-" ],
			strtolower( $name ) );
	}

	protected function sanitize_tag_name( string $tag_name ): string {
		return str_replace( [ ",", ".", ";", ":" ], "", $tag_name );
	}

	/**
	 * Parses image urls from response data and returns them in array.
	 *
	 * @param array $ad_response_data
	 *
	 * @return array
	 */
	protected function parse_image_urls_from_ad_response( array $ad_response_data ): array {
		$image_urls = [];
		foreach ( $ad_response_data['images'] as $image ) {
			if ( $image['large'] ?? null ) {
				$image_urls[] = $image['large']['url'];
			} elseif ( $image['medium'] ?? null ) {
				$image_urls[] = $image['medium']['url'];
			}
		}

		return $image_urls;
	}

	/**
	 * Determines if given NettiX ad already has product stored to WooCommerce and returns that product object.
	 *
	 * @param int $nettix_id
	 *
	 * @return WC_Product|null
	 */
	protected function get_existing_product( int $nettix_id ): ?WC_Product {
		global $wpdb;

		$query  = 'SELECT * FROM ' . $this->database_settings->table_product_map . ' WHERE nettix_id = %d ORDER BY id DESC LIMIT 1';
		$id_map = $wpdb->get_results( $wpdb->prepare( $query, $nettix_id ),
			ARRAY_A );

		if ( count( $id_map ) < 1 ) {
			return null;
		}

		$query    = new WC_Product_Query( [ 'include' => [ (int) $id_map[0]['post_id'] ] ] );
		$products = $query->get_products();

		return $products[0];
	}


	/**
	 * Stores new mapping of NettiX ad ID to WooCommerce product/post ID
	 *
	 * @param int $productId
	 * @param int $nettixId
	 */
	private function add_id_map( int $productId, int $nettixId ) {
		global $wpdb;
		$wpdb->insert( $this->database_settings->table_product_map,
			[ 'post_id' => $productId, 'nettix_id' => $nettixId ] );
	}

	/**
	 * Creates new external product to WooCommerce if one doesn't already exist. If product already exists the existing product is updated with given data.
	 *
	 * @param Ad_Data $ad_data
	 *
	 * @return void
	 */
	protected function create_or_update_product( Ad_Data $ad_data ): void {
		if ( $product = $this->get_existing_product( $ad_data->get_nettix_id() ) ) {
			$this->update_product( $product,
				$ad_data->get_nettix_id(),
				$ad_data->get_title(),
				$ad_data->get_description(),
				$ad_data->get_price(),
				$ad_data->get_image_urls(),
				$ad_data->get_category_ids(),
				$ad_data->get_tag_ids() );
		} else {
			$this->create_product( $ad_data->get_slug_prefix(),
				$ad_data->get_nettix_id(),
				$ad_data->get_product_url(),
				$ad_data->get_title(),
				$ad_data->get_description(),
				$ad_data->get_price(),
				$ad_data->get_image_urls(),
				$ad_data->get_category_ids(),
				$ad_data->get_tag_ids() );
		}
	}

	/**
	 * @param array $ad_response_data
	 *
	 * @return Ad_Data
	 * @throws Exception
	 */
	protected function populate_ad_data( array $ad_response_data ): Ad_Data {
		$ad_data = new Ad_Data();

		if ( ! $nettix_id = (int) ( sanitize_text_field( $ad_response_data['id'] ?? '' ) ) ) {
			throw new Exception( "No id in ad data" );
		}

		if ( ! $product_url = esc_url_raw( $ad_response_data['adUrl'] ?? '' ) ) {
			throw new Exception( "No ad URL" );
		}

		$ad_data->set_nettix_id( $nettix_id );
		$ad_data->set_product_url( $product_url );

		$seller_name_tag = $this->get_tag( sanitize_text_field( $ad_response_data['userName'] ?? '' ) );

		if ( $seller_name_tag ) {
			$ad_data->add_tag_id( $seller_name_tag->term_id );
		}

		if ( ! $maker = sanitize_text_field( $ad_response_data['make']['name'] ?? '' ) ) {
			throw new Exception( "No maker name" );
		}

		if ( $maker_tag = $this->get_tag( $maker ) ) {
			$ad_data->add_tag_id( $maker_tag->term_id );
		}

		$model = sanitize_text_field( $ad_response_data['model']['name'] ?? '' );

		if ( $model === '' ) {
			$model = sanitize_text_field( $ad_response_data['model'] ?? '' );
		}

		$model_type = sanitize_text_field( $ad_response_data['modelType']['name'] ?? '' );

		if ( $model_type === '' ) {
			$model_type = sanitize_text_field( $ad_response_data['modelTypeName'] ?? '' );
		}

		$year = sanitize_text_field( $ad_response_data['year'] ?? '' );

		$title = $maker;

		if ( $model !== '' ) {
			$title .= ' ' . $model;
		}

		if ( $model_type !== '' ) {
			$title .= ' ' . $model_type;
		}

		if ( $year !== '' ) {
			$title .= ' ' . $year;
		}

		$ad_data->set_title( $title );

		$description = sanitize_textarea_field( $ad_response_data['description'] ?? '' );

		if ( $description === '' ) {
			$description = $title;
		}

		$ad_data->set_description( $description );

		$ad_data->set_price( (float) sanitize_text_field( $ad_response_data['price'] ) ?? 0.0 );
		$ad_data->set_image_urls( $this->parse_image_urls_from_ad_response( $ad_response_data ) );

		return $ad_data;
	}

	/**
	 * Creates new WooCommerece product based on ad data fetched from NettiX.
	 * Type of created product is 'external'.
	 *
	 * @param string $slug_prefix
	 * @param int $nettix_id
	 * @param string $product_url
	 * @param string $title
	 * @param string $description
	 * @param float $price
	 * @param array $image_urls_array
	 * @param array $category_ids
	 * @param array $tag_ids
	 */
	protected function create_product(
		string $slug_prefix,
		int $nettix_id,
		string $product_url,
		string $title,
		string $description,
		float $price = 0.0,
		array $image_urls_array = [],
		array $category_ids = [],
		array $tag_ids = []
	) {
		$product = new WC_Product_External();
		$product->set_name( $title );
		$product->set_regular_price( $price );
		$product->set_product_url( $product_url );
		$product->set_button_text( 'Avaa' );
		$product->set_status( 'publish' );
		$product->set_slug( $slug_prefix . '-' . $nettix_id );
		$product->set_description( $description );
		$product->set_category_ids( $category_ids );
		$product->set_tag_ids( $tag_ids );

		$product->save();

		$this->add_id_map( $product->get_id(), $nettix_id );

		$this->add_images_to_product( $product, $nettix_id, $image_urls_array );
	}

	/**
	 * Updates existing WooCommerce product based on data fetched from NettiX.
	 *
	 * @param WC_Product $product
	 * @param int $nettixId
	 * @param string $title
	 * @param string $description
	 * @param float $price
	 * @param array $image_urls_array
	 * @param array $category_ids
	 * @param array $tagIds
	 */
	protected function update_product(
		WC_Product $product,
		int $nettixId,
		string $title,
		string $description,
		float $price = 0.0,
		array $image_urls_array = [],
		array $category_ids = [],
		array $tagIds = []
	) {
		$product->set_name( $title );
		$product->set_regular_price( $price );
		$product->set_description( $description );
		$product->set_category_ids( $category_ids );
		$product->set_tag_ids( $tagIds );
		$product->save();

		// Check if new images were added to product
		$insert_image_urls       = [];
		$product_image_filenames = $this->get_product_image_filenames( (int) $product->get_id() );

		foreach ( $image_urls_array as $imageUrl ) {
			if ( ! in_array( basename( $imageUrl ), $product_image_filenames ) ) {
				$insert_image_urls[] = $imageUrl;
			}
		}

		$this->add_images_to_product( $product, $nettixId, $insert_image_urls );
	}

	/**
	 * Initiates asynchronous downloading and attaching of ad images to WooCommerce product.
	 *
	 * @param WC_Product $product
	 * @param int $nettix_id
	 * @param array $image_urls
	 *
	 * @see Fetcher::wc_integration_nettix_fetcher_ajax_attach_ad_images
	 */
	private function add_images_to_product( WC_Product $product, int $nettix_id, array $image_urls ): void {
		if ( count( $image_urls ) > 0 ) {
			$serialized_product = base64_encode( serialize( $product ) );
			$batch_urls         = array_chunk( $image_urls, 2 );

			foreach ( $batch_urls as $key => $image_urls ) {
				$serialized_data = serialize( $image_urls );
				$nonce           = wp_create_nonce( hash( 'sha256', $product->get_id() . $serialized_data ) );
				$args            = [
					'body'      => [
						'action'     => 'wc_integration_nettix_attach_ad_images',
						'nonce'      => $nonce,
						'nettix_id'  => $nettix_id,
						'product_id' => $product->get_id(),
						'image_urls' => base64_encode( $serialized_data ),
						'product'    => $serialized_product,
						'batch_nbr'  => $key + 1
					],
					'timeout'   => 20,
					'blocking'  => false,
					'cookies'   => [],
					'sslverify' => apply_filters( 'https_local_ssl_verify', false ),
				];
				wp_remote_post( get_site_url() . '/wp-admin/admin-ajax.php', $args );
				usleep( 100000 ); // wait for 200ms before doing next call to avoid congesting the server with requests.
			}
		}
	}

	/**
	 * @param int $product_id
	 *
	 * @return array
	 */
	protected function get_product_image_filenames( int $product_id ): array {
		global $wpdb;

		$query  = "SELECT image_filename FROM " . $this->database_settings->table_product_image_map . " WHERE post_id = %d ORDER BY id ASC";
		$result = $wpdb->get_results( $wpdb->prepare( $query, $product_id ), ARRAY_A );

		$ret = [];
		foreach ( $result as $res ) {
			$ret[] = $res['image_filename'];
		}

		return $ret;
	}

	/**
	 * Removes existing WooCommerce product if ad related to it in NettiX is in state unpublished.
	 *
	 * @param array $adResponseData
	 */
	protected function remove_product( array $adResponseData ) {
		$product = $this->get_existing_product( (int) $adResponseData['id'] );

		if ( $product ) {
			$productId = $product->get_id();
			$product->delete( true );
			$this->remove_product_image_map_entry( $productId );
			$this->remove_product_map_entry( $productId );
		}
	}

	/**
	 * @param int $post_id
	 *
	 * @return void
	 */
	private function remove_product_image_map_entry( int $post_id ) {
		global $wpdb;

		$wpdb->delete( $this->database_settings->table_product_image_map, [ 'post_id' => $post_id ] );
	}

	/**
	 * @param int $post_id
	 *
	 * @return void
	 */
	private function remove_product_map_entry( int $post_id ) {
		global $wpdb;

		$wpdb->delete( $this->database_settings->table_product_map, [ 'post_id' => $post_id ] );
	}
}