<?php

namespace WC_Integration_NettiX\Plugin;

use Exception;
use WC_Integration_NettiX\NettiX_Client\Base_Client;
use WC_Integration_NettiX\NettiX_Client\Nettiauto_Client;
use WC_Integration_NettiX\NettiX_Client\Nettimoto_Client;
use WC_Integration_NettiX\NettiX_Client\Nettivene_Client;
use WC_Product;
use WC_Product_External;
use WP_Error;

class Fetcher {
	private Plugin_Settings $settings;
	private Database_Settings $database_settings;

	public function __construct( Plugin_Settings $settings ) {
		$this->settings          = $settings;
		$this->database_settings = new Database_Settings();
	}

	/**
	 * Fetches all ads from NettiX. Loops through all enabled services.
	 */
	public function fetch() {
		if ( $this->settings->get_service_bike() == 'yes' ) {
			$client  = new Nettimoto_Client( $this->settings );
			$handler = new Nettimoto_Ad_Handler();
			$this->fetch_ads( $client, $handler );
		}

		if ( $this->settings->get_service_boat() == 'yes' ) {
			$client  = new Nettivene_Client( $this->settings );
			$handler = new Nettivene_Ad_Handler();
			$this->fetch_ads( $client, $handler );
		}

		if ( $this->settings->get_service_car() == 'yes' ) {
			$client  = new Nettiauto_Client( $this->settings );
			$handler = new Nettiauto_Ad_Handler();
			$this->fetch_ads( $client, $handler );
		}
	}

	/**
	 * Start fetching and handling ads of one NettiX service client has access to.
	 *
	 * @param Base_Client $client
	 * @param Ad_Handler $handler
	 *
	 * @return void
	 */
	private function fetch_ads( Base_Client $client, Ad_Handler $handler ): void {
		$criteria = [];

        try {
            if ($ad_count = $client->get_ad_count($criteria)) {
                $page_count = ceil($ad_count / 100);
                for ($i = 1; $i < $page_count + 1; $i++) {
                    $criteria['rows'] = 100;
                    $criteria['page'] = $i;
                    try {
                        $handler->handle($client->search($criteria));
                    } catch (Exception $e) {
                        return;
                    }
                }
            }
        } catch (Exception $e) {
            wc_get_logger()->error("NettiX API: {$e->getMessage()}");
        }
	}

	/**
	 * Asynchronously handles downloading ad images from NettiX and attaching them to WooCommerce products
	 */
	public function wc_integration_nettix_fetcher_ajax_attach_ad_images() {
		session_write_close();

		$nonce      = sanitize_text_field( $_POST['nonce'] );
		$image_urls = sanitize_text_field( $_POST['image_urls'] );
		$product_id = (int) sanitize_text_field( $_POST['product_id'] );
		$product    = sanitize_text_field( $_POST['product'] );
		$nettix_id  = (int) sanitize_text_field( $_POST['nettix_id'] );

		$serialized_image_urls = base64_decode( sanitize_text_field( $image_urls ), true );

		if ( ! wp_verify_nonce( $nonce, hash( 'sha256', $product_id . $serialized_image_urls ) ) ) {
			wp_die();
		}

		$product = unserialize( base64_decode( $product, true ) );
		if ( ! $product instanceof WC_Product_External ) {
			wp_die();
		}

		$image_urls = unserialize( $serialized_image_urls );

		if ( ! is_array( $image_urls ) ) {
			wp_die();
		}

		foreach ( $image_urls as $image_url ) {
			$image_url      = esc_url( sanitize_text_field( $image_url ), [ 'http', 'https' ], 'download' );
			$image_filename = basename( $image_url );

			$file = wc_rest_upload_image_from_url( $image_url );

			if ( $file instanceof WP_Error ) {
				continue;
			}

			if ( is_array( $file ) ) {
				$imageId = wc_rest_set_uploaded_image_as_attachment( $file, $product_id );
				$this->create_product_image_map( $product_id, $nettix_id, $imageId, $image_filename );
				$this->update_product_images( $product );
			}
		}

		wp_die();
	}

	/**
	 * @param int $product_id
	 * @param int $nettix_id
	 * @param int $image_id
	 * @param string $image_filename
	 *
	 * @return void
	 */
	private function create_product_image_map(
		int $product_id,
		int $nettix_id,
		int $image_id,
		string $image_filename
	): void {
		global $wpdb;

		$wpdb->insert( $this->database_settings->table_product_image_map,
			[
				'post_id'        => $product_id,
				'nettix_id'      => $nettix_id,
				'image_id'       => $image_id,
				'image_filename' => $image_filename
			] );
	}

	/**
	 * @param WC_Product $product
	 *
	 * @return void
	 */
	private function update_product_images( WC_Product $product ): void {
		$image_ids = $this->get_product_image_ids( $product->get_id() );

		if ( ! $product->get_image_id() ) {
			$product->set_image_id( array_shift( $image_ids ) );
		}

		$product->set_gallery_image_ids( $image_ids );
		$product->save();
	}

	/**
	 * @param int $product_id
	 *
	 * @return array
	 */
	private function get_product_image_ids( int $product_id ): array {
		global $wpdb;

		$query  = "SELECT image_id FROM " . $this->database_settings->table_product_image_map . " WHERE post_id = %d ORDER BY id ASC";
		$result = $wpdb->get_results( $wpdb->prepare( $query, $product_id ), ARRAY_A );

		$ret = [];
		foreach ( $result as $res ) {
			$ret[] = $res['image_id'];
		}

		return $ret;
	}
}